from django.http import request
from django.shortcuts import render, HttpResponse, redirect
from django.contrib.auth.models import User, auth
from django.contrib import messages
from django.core.mail import send_mail
from django.contrib.auth import authenticate, login, logout
from django_email_verification import sendConfirm
from django.contrib.auth.decorators import login_required
from django.views.decorators.cache import cache_control
from accounts.views import userTypeCheck
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import reverse, reverse_lazy
from django.views import generic
from django.views.generic.edit import CreateView, DeleteView, UpdateView
from gyans.models import Answer, Comment, Question, MyQuestion, UpVote, DownVote
from django.utils.decorators import method_decorator
from django.views.decorators.cache import never_cache
from django.http import JsonResponse
import re

decorators = [never_cache,]

# Create your views here.
@cache_control(no_cache=True, must_revalidate=True, no_store=True)
def home(request):
    """View for the home page when user is not logged in, otherwise redirect to the correct dashboard."""
    if request.user.is_authenticated:
        email = str(request.user.email)
        if(userTypeCheck(email) == "IITian"):
            
            return redirect("/iitiandashboard")
        
        return redirect("/normaldashboard")

    return render(request, "index.html")


@login_required(login_url='/login')
@cache_control(no_cache=True, must_revalidate=True, no_store=True)
def iitiandashboard(request):
    """View for the dashboard for the IITian user."""
    if request.user.is_authenticated:
        email = str(request.user.email)
        if(userTypeCheck(email) == "IITian"):
            
            return render(request, "gyans/iitiandashboard.html")
        else:
            return redirect("/normaldashboard")


@login_required(login_url='/login')
@cache_control(no_cache=True, must_revalidate=True, no_store=True)
def normaldashboard(request):
    """View for the dashboard for the normal user."""
    if request.user.is_authenticated:
        email = str(request.user.email)
        if(userTypeCheck(email) == "normal"):
            
            return render(request, "gyans/normaldashboard.html")
        else:
            return redirect("/iitiandashboard")


@cache_control(no_cache=True, must_revalidate=True, no_store=True)
def about(request):
    """View for the about page when user is not logged in."""
    return render(request, "about.html")

# -------------------------------class based views-------------------------

@method_decorator(decorators, name='dispatch')
class QuestionCreate(LoginRequiredMixin, CreateView):
    """View for the Question Form only for the normal user."""
    model = Question,MyQuestion
    fields = ['question_text']

    def get(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            email = str(request.user.email)
            if(userTypeCheck(email) == "IITian"):
                return redirect("/iitiandashboard")
            else:
                return render(request, 'gyans/question_form.html')

    def post(self, request, *args, **kwargs):
        question_text = request.POST.get('question_text')
        author = self.request.user.username
        question = Question.objects.create(
            author=author, question_text=question_text)
        MyQuestion.objects.create(question=question)
        response = redirect('/questions')
        return response

@method_decorator(decorators, name='dispatch')
class QuestionListView(generic.ListView):
    """View representing all questions in the system."""
    model = Question
    #paginate_by = 10

@method_decorator(decorators, name='dispatch')
class MyQuestionListView(generic.ListView):
    """View representing questions asked by the particular user, it is only for normal user."""
    model = MyQuestion
    #paginate_by = 10

@method_decorator(decorators, name='dispatch')
class QuestionDetailView(generic.DetailView):
    """Generic class-based detail view for a Question."""
    model = Question

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        question_id = self.kwargs['pk']
        question = Question.objects.get(id=question_id)
        username = Question.objects.get(id=question_id).author
        answer_list = Answer.objects.filter(question=question)
        comment_dictionary = {
            ans.id: Comment.objects.filter(answer=ans) for ans in answer_list
        }
        context['type'] = userTypeCheck(self.request.user.email)
        context['answer_list'] = answer_list
        context['answer_url'] = '/question/'+str(question_id)+'/answer/'
        context['upvote_url'] = '/answer/upvote/'
        context['downvote_url'] = '/answer/downvote/'
        context['comment_dictionary'] = comment_dictionary
        return context

@method_decorator(decorators, name='dispatch')
class UpdateQuestion(LoginRequiredMixin, UpdateView):
    """Generic class-based detail view for updating Question."""
    model = Question
    fields = ['question_text']
    success_url = reverse_lazy('questions')
    template_name = 'gyans/question_update_form.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        question_id = self.kwargs['pk']
        question = Question.objects.get(id=question_id)
        context['question_text'] = question.question_text
        return context

@method_decorator(decorators, name='dispatch')
class AnswerCreate(LoginRequiredMixin, CreateView):
    """View to create answer form, it is only for IITian user."""
    model = Answer
    fields = ['answer_text']

    def get(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            email = str(request.user.email)
            if(userTypeCheck(email) == "normal"):
                return redirect("/normaldashboard")
            else:
                q_id = self.kwargs['pk']
                return render(request, 'gyans/answer_form.html', {"q_id": q_id})

    def post(self, request, *args, **kwargs):
        answer_text = request.POST.get('answer_text')
        answer = Answer.objects.create(author=self.request.user.username, question=Question.objects.get(
                id=self.kwargs['pk']), answer_text=answer_text)
        response = redirect(
            reverse('question-detail', kwargs={'pk': answer.question.id}))
        return response

@method_decorator(decorators, name='dispatch')
class UpdateAnswer(LoginRequiredMixin, UpdateView):
    """View to create answer form to update it, it is only for IITian user."""
    model = Answer
    fields = ['answer_text']
    template_name = 'gyans/answer_update_form.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        answer_id = self.kwargs['pk']
        answer = Answer.objects.get(id=answer_id)
        self.pk = answer.question.id
        context['answer_text'] = answer.answer_text
        return context

    def get_success_url(self):
        return (reverse('question-detail', kwargs={'pk': self.object.question.id}))



@method_decorator(decorators, name='dispatch')
class CommentCreate(LoginRequiredMixin, CreateView):
    """View to create comment form."""
    model = Comment
    fields = ['author', 'answer', 'comment_text']

    def get(self, request, *args, **kwargs):
        a_id = self.kwargs['pk']
        return render(request, 'gyans/comment_form.html', {"a_id": a_id})

    def post(self, request, *args, **kwargs):
        comment_text = request.POST.get('comment_text')
        params = self.kwargs['pk']
        author = self.request.user
        answer = Answer.objects.get(id=params)
        Comment.objects.create(
            author=author, answer=answer, comment_text=comment_text)
        response = redirect(
            reverse('question-detail', kwargs={'pk': answer.question.id}))
        return response


@method_decorator(decorators, name='dispatch')
class UpvoteCreate(LoginRequiredMixin, CreateView):
    """View to Upvote the Answer."""
    model = Answer, UpVote
    fields = ['answer_text', 'id', 'upvote']

    def post(self, request, *args, **kwargs):
        
        if request.is_ajax():
            answer_id = self.kwargs['pk']
            answer = Answer.objects.get(id=answer_id)
            vote, created = UpVote.objects.get_or_create(author=answer.author, answerId=answer_id)
            if created:
                answer.upvote += 1
            else:
                answer.upvote = answer.upvote
            answer.save()
            print(request.is_ajax())
            response=JsonResponse({'message':answer.upvote},status=200)
        else:
            
            answer_id = self.kwargs['pk']
            answer = Answer.objects.get(id=answer_id)
            vote, created = UpVote.objects.get_or_create(author=answer.author, answerId=answer_id)
            if created:
                answer.upvote += 1
            else:
                answer.upvote = answer.upvote
            answer.save()
            print(request.is_ajax())
            response = redirect(
                reverse('question-detail', kwargs={'pk': answer.question.id}))
        return response


@method_decorator(decorators, name='dispatch')
class DownvoteCreate(LoginRequiredMixin, CreateView):
    """View to Downvote the Answer."""
    model = Answer, DownVote
    fields = ['answer_text', 'id', 'downvote']

    def post(self, request, *args, **kwargs):
        
        if not request.is_ajax():
            
            answer_id = self.kwargs['pk']
            answer = Answer.objects.get(id=answer_id)
            vote, created = DownVote.objects.get_or_create(author=answer.author, answerId=answer_id)
            if created:
                answer.downvote += 1
            else:
                answer.downvote = answer.downvote
            answer.save()
            response = redirect(
                reverse('question-detail', kwargs={'pk': answer.question.id}))
        else:
            
            answer_id = self.kwargs['pk']
            answer = Answer.objects.get(id=answer_id)
            vote, created = DownVote.objects.get_or_create(author=answer.author, answerId=answer_id)
            if created:
                answer.downvote += 1
            else:
                answer.downvote = answer.downvote
            answer.save()
            response=response=JsonResponse({'message':answer.downvote},status=200)
        return response


@method_decorator(decorators, name='dispatch')
class AnswerListView(generic.ListView):
    """View to list all the Answers, answered by the particular user. it is used only for IITian User."""
    model = Answer
    paginate_by = 3

